package com.proximity.demo.eventbriteApi.Events;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.proximity.demo.eventbriteApi.Categories.Category;
import com.proximity.demo.eventbriteApi.Subcategories.Subcategory;
import com.proximity.demo.eventbriteApi.Venues.Venue;

import javax.persistence.*;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

@Entity
@JsonIgnoreProperties(ignoreUnknown = true)
@Table(name="EVENT")
public class Event {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private long eventId;

    private long id;
    @Column(unique = true)
    private String eventName;
    @Column(length = 10000)
    private String eventDescription;
    private long category_id;
    private long subcategory_id;
    private long venue_id;
    private LocalDateTime startDate;
    private LocalDateTime endDate;
    private String status;
    private Date created;
    private String resource_url;
    private String logo;

    public Event() { }

    public Event(String eventName, String eventDescription, long category_id, long subcategory_id, long venue_id,
                 LocalDateTime startDate, LocalDateTime endDate, String status, Date created, String resource_url,
                 String logo, Set<Venue> venue) {
        this.eventName = eventName;
        this.eventDescription = eventDescription;
        this.category_id = category_id;
        this.subcategory_id = subcategory_id;
        this.venue_id = venue_id;
        this.startDate = startDate;
        this.endDate = endDate;
        this.status = status;
        this.created = created;
        this.resource_url = resource_url;
        this.logo = logo;
        this.venue = venue;
    }

    @JsonProperty("name")
    private void unpackName(Map<String, Object> name) {
        this.eventName = (String) name.get("text");
    }

    @JsonProperty("description")
    private void unpackDescription(Map<String, Object> description) {
        this.eventDescription = (String) description.get("text");
    }

    @JsonProperty("start")
    private void unpackStart(Map<String, Object> start) {
        this.startDate = LocalDateTime.parse((String) start.get("local"));
    }

    @JsonProperty("end")
    private void unpackEnd(Map<String, Object> end) {
        this.endDate =  LocalDateTime.parse((String) end.get("local"));
    }

    @JsonProperty("logo")
    private void unpackLogo(Map<String, Object> logo) {
        this.logo = (String) logo.get("url");
    }

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name="event_venue",
            joinColumns=@JoinColumn(name="event_id"),
            inverseJoinColumns=@JoinColumn(name="venue_id"))
    private Set<Venue> venue = new HashSet<>();

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name="event_category",
    joinColumns = @JoinColumn(name="category_id"),
    inverseJoinColumns=@JoinColumn(name="event_id"))
    private Set<Category> category = new HashSet<>();

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name="Event_subcategory",
    joinColumns = @JoinColumn(name="subcategory_id"),
    inverseJoinColumns = @JoinColumn(name="event_id"))
    private Set<Subcategory> subcategory = new HashSet<>();


    public long getId() { return id; }

    public void setId(long id) { this.id = id; }

    public String getEventName() { return eventName; }

    public void setEventName(String eventName) { this.eventName = eventName; }

    public String getEventDescription() { return eventDescription; }

    public void setEventDescription(String eventDescription) { this.eventDescription = eventDescription; }

    public long getCategory_id() { return category_id; }

    public void setCategory_id(long category_id) { this.category_id = category_id; }

    public long getSubcategory_id() { return subcategory_id; }

    public void setSubcategory_id(long subcategory_id) { this.subcategory_id = subcategory_id; }

    public long getVenue_id() { return venue_id; }

    public void setVenue_id(long venue_id) { this.venue_id = venue_id; }

    public LocalDateTime getStartDate() { return startDate; }

    public void setStartDate(LocalDateTime startDate) { this.startDate = startDate; }

    public LocalDateTime getEndDate() { return endDate; }

    public void setEndDate(LocalDateTime endDate) { this.endDate = endDate; }

    public String getStatus() { return status; }

    public void setStatus(String status) { this.status = status; }

    public Date getCreated() { return created; }

    public void setCreated(Date created) { this.created = created; }

    public String getResource_url() { return resource_url; }

    public void setResource_url(String resource_url) { this.resource_url = resource_url; }

    public String getLogo() { return logo; }

    public void setLogo(String logo) { this.logo = logo; }

    public Set<Venue> getVenue() { return venue; }

    public void setVenue(Set<Venue> venue) { this.venue = venue; }

    public Set<Category> getCategory() { return category; }

    public void setCategory(Set<Category> category) { this.category = category; }

    @Override
    public String toString() {
        return "Event{" +
                "id=" + id +
                ", eventName='" + eventName + '\'' +
                ", eventDescription='" + eventDescription + '\'' +
                ", category_id=" + category_id +
                ", subcategory_id=" + subcategory_id +
                ", venue_id=" + venue_id +
                ", startDate=" + startDate +
                ", endDate=" + endDate +
                ", status='" + status + '\'' +
                ", created=" + created +
                ", resource_url='" + resource_url + '\'' +
                ", logo='" + logo + '\'' +
                ", venue=" + venue +
                '}';
    }

    public Set<Subcategory> getSubcategory() {
        return subcategory;
    }


    public void setSubcategory(Set<Subcategory> subcategory) {
        this.subcategory = subcategory;
    }
}
