package com.proximity.demo.eventbriteApi.Venues;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.proximity.demo.eventbriteApi.Events.Event;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

@Entity
@JsonIgnoreProperties(ignoreUnknown = true)
@Table(name="VENUE")
public class Venue implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long venueid;

    @Column(unique = true)
    private long id;
    private String name;
    private String address_1;
    private String address_2;
    private String city;
    private String region;
    private String postal_code;
    private String country;
    private String latitude;
    private String longitude;
    private String localized_multi_line_address_display;
    private String resource_uri;
    private int age_restriction;
    private int capacity;

    public Venue() { }

    public Venue(String name, String address_1, String address_2, String city, String region,
                 String postal_code, String country, String latitude, String longitude,
                 String localized_multi_line_address_display, String resource_uri, int age_restriction,
                 int capacity, Set<Event> event) {
        this.name = name;
        this.address_1 = address_1;
        this.address_2 = address_2;
        this.city = city;
        this.region = region;
        this.postal_code = postal_code;
        this.country = country;
        this.latitude = latitude;
        this.longitude = longitude;
        this.localized_multi_line_address_display = localized_multi_line_address_display;
        this.resource_uri = resource_uri;
        this.age_restriction = age_restriction;
        this.capacity = capacity;
        this.event = event;
    }


    @JsonProperty("address")
    private void unpackAddress(Map<String, Object> address) {
        this.address_1 = (String) address.get("address_1");
        this.address_2 = (String) address.get("address_2");
        this.city = (String) address.get("city");
        this.region = (String) address.get("region");
        this.postal_code = (String)address.get("postal_code");
        this.country = (String) address.get("country");
    }

    @ManyToMany(mappedBy = "venue", fetch = FetchType.EAGER)
    private Set<Event> event = new HashSet<>();

    public Long getId() { return id; }

    public void setId(Long id) { this.id = id; }

    public String getName() { return name; }

    public void setName(String name) { this.name = name; }

    public String getAddress_1() { return address_1; }

    public void setAddress_1(String address_1) { this.address_1 = address_1; }

    public String getAddress_2() { return address_2; }

    public void setAddress_2(String address_2) { this.address_2 = address_2; }

    public String getCity() { return city; }

    public void setCity(String city) { this.city = city; }

    public String getRegion() { return region; }

    public void setRegion(String region) { this.region = region; }

    public String getPostal_code() { return postal_code; }

    public void setPostal_code(String postal_code) { this.postal_code = postal_code; }

    public String getCountry() { return country; }

    public void setCountry(String country) { this.country = country; }

    public String getLatitude() { return latitude; }

    public void setLatitude(String latitude) { this.latitude = latitude; }

    public String getLongitude() { return longitude; }

    public void setLongitude(String longitude) { this.longitude = longitude; }

    public String getLocalized_multi_line_address_display() { return localized_multi_line_address_display; }

    public void setLocalized_multi_line_address_display(String localized_multi_line_address_display) {
        this.localized_multi_line_address_display = localized_multi_line_address_display;
    }

    public String getResource_uri() { return resource_uri; }

    public void setResource_uri(String resource_uri) { this.resource_uri = resource_uri; }

    public int getAge_restriction() { return age_restriction; }

    public void setAge_restriction(int age_restriction) { this.age_restriction = age_restriction; }

    public int getCapacity() { return capacity; }

    public void setCapacity(int capacity) { this.capacity = capacity; }

    public Set<Event> getEvent() { return event; }

    public void setEvent(Set<Event> event) { this.event = event; }

    @Override
    public String toString() {
        return "Venue{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", address_1='" + address_1 + '\'' +
                ", address_2='" + address_2 + '\'' +
                ", city='" + city + '\'' +
                ", region='" + region + '\'' +
                ", postal_code=" + postal_code +
                ", country='" + country + '\'' +
                ", latitude='" + latitude + '\'' +
                ", longitude='" + longitude + '\'' +
                ", localized_multi_line_address_display='" + localized_multi_line_address_display + '\'' +
                ", resource_uri='" + resource_uri + '\'' +
                ", age_restriction=" + age_restriction +
                ", capacity=" + capacity +
                ", event=" + event +
                '}';
    }
}
